/*******************************************************************************
* RTCC.c
* Driver functions used for RTC module.
* ------------------------------------------------------------------------------
*
*******************************************************************************/
#include "RTCC.h"
#include <built_in.h>
#include "__Time.h"
#include "Buffers.h"
#include "UART1.h"
#include "Common.h"




/*******************************************************************************
* RTC counter mode example
*******************************************************************************/


// Variables
unsigned long RTC_Time, old_RTC_Time;
unsigned long RTC_Date, old_RTC_Date;

unsigned short RTC_Timer;
unsigned char RTC_State = 0;







/*******************************************************************************
* Hibernate Interrupt: not used
*******************************************************************************/
void Hibernate_Interrupt() iv IVT_INT_HIBERNATE ics ICS_AUTO {
     HIB_IC_RTCALT0_bit = 1;  // Clear RTCALT0 interrupt
}

/*******************************************************************************
* Function HibernateRTCDisable()
*******************************************************************************/
void HibernateRTCDisable(void) {
     while(!HIB_CTL_WRC_bit);
     HIB_CTL_RTCEN_bit = 0;
     //while(!HIB_CTL_WRC_bit);
     //HIB_CTL_CLK32EN_bit = 0;
}


/*******************************************************************************
* Function HibernateRTCEnable()
* ------------------------------------------------------------------------------
* Overview: Function enables hibernate module
* Input: Nothing
* Output: Nothing
* Returns: Nothing
*******************************************************************************/
void HibernateRTCEnable(void) {
     //while(!HIB_CTL_WRC_bit);
     //HIB_CTL_RTCEN_bit = 0;

     while(!HIB_CTL_WRC_bit);
     //HIB_CTL = 0x40;              // Enable 32.768kHz osc
     HIB_CTL_CLK32EN_bit = 1;     // Enable 32.768kHz oscillator
     Delay_ms(1000);

     while(!HIB_CTL_WRC_bit);
     HIB_RTCM0 = 86400;           // Match with counter when 24 hours have elapsed

     while(!HIB_CTL_WRC_bit);
     //HIB_IM = 0x01;               // Set interrupt mask
     HIB_IM_RTCALT0_bit = 1;      // Allow interrupts when HIBERTCM0 register matches HIBERTCC register

     while(!HIB_CTL_WRC_bit);
     //HIB_CTL = 0x41;              // Turn on counter
     HIB_CTL_RTCEN_bit = 1;        // Turn on counter
}

/*******************************************************************************
* Function HibernateRTCSet()
* ------------------------------------------------------------------------------
* Overview: Function sets desired time into hibernate registers
* Input: Pointer to a time structure containing hours, minutes and seconds
* Output: Nothing
* Returns: Nothing
*******************************************************************************/
void HibernateRTCSet(TimeStruct *temp) {
     unsigned long tmptmp = 0;
     char *BufNum;

     //HibernateRTCDisable();
     tmptmp  = (unsigned long)temp->hh*3600;     // Convert hours to seconds
     tmptmp += (unsigned long)temp->mn*60;       // Convert minutes to seconds and add to total
     tmptmp += (unsigned long)temp->ss;          // Add in seconds to total

     while(!HIB_CTL_WRC_bit);
     HIB_RTCLD = tmptmp;                         // Set RTC counter

/*
     BufNum = GetBuffer();
     LongWordToStr(tmptmp, BufNum);
     strcat(BufNum,"\r\n");
     //sprintf(BufNum,"hh: %d  mn: %d  ss: %d\r\n",temp->hh,temp->mn,temp->ss);
     UART1_PutQueue(BufNum);
*/
}

/*******************************************************************************
* Function HibernateRTCGet()
* ------------------------------------------------------------------------------
* Overview: Reads time from register
* Input: Pointer to time structure
* Output: Hours, minutes, and seconds saved in user time structure
* Returns: Current counter value
*******************************************************************************/
unsigned long HibernateRTCGet(TimeStruct *temp) {
     unsigned long clk;
     char *BufNum;

     clk = HIB_RTCC;                   // Get RTC counter
     temp->hh = clk%86400/3600;        // Extract hours and put in time structure
     temp->mn = clk%3600/60;           // Extract minutes and put in time structure
     temp->ss = clk%60;                // Extract seconds and put in time structure
/*
     BufNum = GetBuffer();
     sprintf(BufNum,"hh: %d  mn: %d  ss: %d\r\n",temp->hh,temp->mn,temp->ss);
     UART1_PutQueue(BufNum);
*/
     return clk;                             // Return current counter value
}




/*******************************************************************************
* Function RTC_Init()
* ------------------------------------------------------------------------------
* Overview: Initializes RTC module with default hours, minutes, and seconds
* Input: Nothing
* Output: Nothing
* Returns: Nothing
*******************************************************************************/
void RTC_Init(void) {
     TimeStruct My_Time;

     My_Time.ss = 12;
     My_Time.mn = 43;
     My_Time.hh = 22;
     My_Time.md = 07;
     My_Time.mo = 4;
     My_Time.yy = 2014;

     HibernateRTCEnable();        // Go enable RTC
     HibernateRTCSet(&My_Time);
}

/*******************************************************************************
* Function Set_RTC(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Sets RTC time and calendar
* Input: Time structure
* Output: Nothing
* Returns: Nothing
*******************************************************************************/
void Set_RTC(TimeStruct *DateTime) {
     TimeStruct My_Time;

     My_Time.ss = DateTime->ss;
     My_Time.mn = DateTime->mn;
     My_Time.hh = DateTime->hh;   // Assume a 24 hour clock

     My_Time.md = DateTime->md;
     My_Time.mo = DateTime->mo;
     My_Time.yy = DateTime->yy;

     HibernateRTCSet(&My_Time);
}

/*******************************************************************************
* Function RTCC_Read(TTime *RTCC_Time)
* ------------------------------------------------------------------------------
* Overview: Reads RTC time and calendar if different from last time
* Input: User time structure
* Output: Puts time and date in user time structure. Puts current count in RTC_Time
* Returns: 0 if counter did not change, 1 if new time saved
*******************************************************************************/
char RTCC_Read(TimeStruct *DateTime) {
     char temp = 0;
     TimeStruct My_Time;

     My_Time.yy = 0;
     My_Time.mo = 0;
     My_Time.md = 0;

     RTC_Time = HibernateRTCGet(&My_Time);

     if (RTC_Time != Old_RTC_Time) {
         DateTime->ss = My_Time.ss;
         DateTime->mn = My_Time.mn;
         DateTime->hh    = My_Time.hh;

         DateTime->md    = My_Time.md;
         DateTime->mo    = My_Time.mo;
         DateTime->yy    = My_Time.yy;

         Old_RTC_Time = RTC_Time;
         temp = 1;
     }
     return temp;
}

/*******************************************************************************
* End of counter mode example
*******************************************************************************/









/*******************************************************************************
* RTC calendar mode example
*******************************************************************************/

// Variables
unsigned long TimeLoad;  // Holds write data for HIBCALLD0 register
unsigned long DateLoad;  // Holds write data for HIBCALLD1 register
unsigned long TimeRead;  // Holds read data from HIBCALLD0 register
unsigned long DateRead;  // Holds read data from HIBCALLD1 register
unsigned Yr;  // Year value
char Mo;      // Month value
char Da;      // Day value
char Hr;      // Hour value
char Mn;      // Minute value
char Se;      // Second value
char DoW;     // Day of week value

extern unsigned char ObtainedTime;  // CC3000 flag
extern unsigned char State;         // CC3000 State machine
extern TimeStruct my_time;          // CC3000 time structure




/******************************************************************************
* Calculate the day-of-the-week using Zeller's Congruence
* Sun = 0, Mon = 1, Tues = 2, Wed = 3, Thur = 4, Fri = 5, Sat = 6
******************************************************************************/
char DayOfWeek(unsigned yr, char mo, char dy) {
     int dow;

     if (mo < 3) {
         mo += 12;
         --yr;
     }

     /* Zeller's congruence - or at least, some form of it. */
     dow = dy + (13 * mo - 27) / 5 + yr + yr/4 - yr/100 + yr/400;
     while (dow < 0) dow += 7;
     dow %= 7;
     return (char)dow;
}

/*******************************************************************************
* RTC Initialize
*******************************************************************************/
void RTC_Initialize(void) {
     HIB_CTL_CLK32EN_bit = 1;            // Enable 32.768kHz oscillator
}

/*******************************************************************************
* RTC State Machine
*******************************************************************************/
void RTC_StateMachine(void) {
     char *BufNum;
     
     
     switch (RTC_State) {
     
         case 0: // Stay in this state until released by user
             break;

     /**********************************************************************************************************************************************/
     /*  Obtain  date and time from web server and set date and time registers in RTC */
         
         case 1:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 2;
             break;
             
         case 2:  // Enable calendar mode
             HIB_CALCTL_CALEN_bit =1;
             RTC_State = 3;
             break;
             
         case 3:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 4;
             break;
             
         case 4:  // Set 24 hour mode
             HIB_CALCTL_CAL24_bit =1;
             RTC_State = 5;
             break;
             
         case 5:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 6;
             break;
             
         case 6:  // Disable RTC run
             HIB_CTL_RTCEN_bit = 0;
             RTC_State = 7;
             break;
         
         case 7:  // Start CC3000 State machine to get date and time
             State = 1;
             RTC_State = 8;
             break;
     
         case 8:  // Hold here until date/time obtained
             if (State != 0 && ObtainedTime != TRUE) {
                 RTC_State = 8;
             } else {
                 if (ObtainedTime == TRUE) {
                     TimeLoad = 0;
                     DateLoad = 0;
                     RTC_State = 9;
                 } else {
                     UART1_PutQueue("Couldn't obtain time. Aborting...\r\n");
                     RTC_State = 0;
                 }
             }
             break;
     
         case 9:  // Prepare time and date load
             TimeLoad = (my_time.hh&0x1F);            // Set hours
             TimeLoad = (TimeLoad<<8);
             
             TimeLoad |= (my_time.mn&0x3F);           // Set minutes
             TimeLoad = (TimeLoad<<8);
             
             TimeLoad |= (my_time.ss&0x3F);           // Set seconds
#if DEBUG
             // Diagnostic
             UART1_PutQueue("Time load value: ");
             BufNum = GetBuffer();
             LongWordToHex(TimeLoad, BufNum);
             strcat(BufNum,"\r\n");
             UART1_PutQueue(BufNum);
#endif
             // Calculate day-of-week using Zeller's Congruence
             DoW = DayOfWeek(((my_time.yy-2000)&0x7F), (my_time.mo&0x0F), (my_time.md&0x1F));
             
             DateLoad = DoW;                          // Set day-of-week
             DateLoad = (DateLoad<<8);
             
             DateLoad |= ((my_time.yy-2000)&0x7F);    // Set year
             DateLoad = (DateLoad<<8);
#if DEBUG
             // Diagnostic
             BufNum = GetBuffer();
             sprintf(BufNum,"Year is: %d\r\n",my_time.yy);
             UART1_PutQueue(BufNum);
#endif
             DateLoad |= (my_time.mo&0x0F);           // Set month
             DateLoad = (DateLoad<<8);

             DateLoad |= (my_time.md&0x1F);           // Set day
#if DEBUG
             // Diagnostic
             UART1_PutQueue("Date load value: ");
             BufNum = GetBuffer();
             LongWordToHex(DateLoad, BufNum);
             strcat(BufNum,"\r\n");
             UART1_PutQueue(BufNum);
#endif
             RTC_State = 10;
             break;
             
         case 10:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 11;
             break;
             
         case 11:  // Write calendar register for time
             HIB_CALLD0 = TimeLoad;
             RTC_State = 12;
             break;
             
         case 12:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 13;
             break;
             
         case 13:  // Write calendar register for date
             HIB_CALLD1 = DateLoad;
             RTC_State = 14;
             break;
     
         case 14:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 15;
             break;
     
         case 15:  // Enable RTC run
             HIB_CTL_RTCEN_bit = 1;
             UART1_PutQueue("RTC set\r\n");
             RTC_State = 0;
             break;
     
     /**********************************************************************************************************************************************/
     /*  Read calendar registers and display */
         
         case 16:  // Read time register
             TimeRead = HIB_CAL0;
             DateRead = HIB_CAL1;

             UART1_PutQueue("Time read: ");
             BufNum = GetBuffer();
             LongWordToHex(TimeRead, BufNum);
             strcat(BufNum,"\r\n");
             UART1_PutQueue(BufNum);
             
             UART1_PutQueue("Date read: ");
             BufNum = GetBuffer();
             LongWordToHex(DateRead, BufNum);
             strcat(BufNum,"\r\n");
             UART1_PutQueue(BufNum);
             
             RTC_State = 17;
             break;
     
         case 17:
             Yr = (char)(DateRead >> 16);
             Yr += 2000;
             Mo = DateRead >> 8;
             Da = DateRead;
             
             Hr = TimeRead >> 16;
             Mn = TimeRead >> 8;
             Se = TimeRead;
             
             BufNum = GetBuffer();
             sprintf(BufNum,"Date: %d/%d/%d  Time: %d:%02d:%02d\r\n",Yr,Mo,Da,Hr,Mn,Se);
             UART1_PutQueue(BufNum);
             RTC_State = 0;
             
             DoW = DayOfWeek(Yr, Mo, Da);
             
             BufNum = GetBuffer();
             sprintf(BufNum,"DoW: %d\r\n",DoW);
             UART1_PutQueue(BufNum);

             RTC_State = 0;
             break;
     
     /**********************************************************************************************************************************************/
     /*  Test: Write calendar registers with fixed data */
     
         case 18:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 19;
             break;

         case 19:  // Enable calendar mode
             HIB_CALCTL_CALEN_bit =1;
             RTC_State = 20;
             break;

         case 20:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 21;
             break;

         case 21:  // Set 24 hour mode
             HIB_CALCTL_CAL24_bit =1;
             RTC_State = 22;
             break;
         
         case 22:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 23;
             break;

         case 23:  // Disable RTC run
             HIB_CTL_RTCEN_bit = 0;
             RTC_State = 24;
             break;
         
         case 24:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 25;
             break;

         case 25:  // Write calendar register for time
             HIB_CALLD0 = 0x000D091C;
             RTC_State = 26;
             break;

         case 26:  // Wait here for write complete
             if (HIB_CTL_WRC_bit)
                 RTC_State = 27;
             break;

         case 27:  // Write calendar register for date
             HIB_CALLD1 = 0x000F0405;
             RTC_State = 28;
             break;
     
         case 28:
             UART1_PutQueue("Test complete\r\n");
             RTC_State = 0;
             break;
     
     }
}
















/*******************************************************************************
* End of calendar mode example
*******************************************************************************/