/******************************************************************************************************************************************
  UART1.c
  This UART module communicates with the FTDI Serial-to-USB FT232RL chip on the Mikromedia 5 for Tiva shield. It's mainly used for
  diagnostic messages and requests for information.
  
  In this version the UART FIFOs are disabled and the hardware functions with only one Data Register.
  
  |        Shield                | |                   Tiva
--|-----------------------|------| |-------------------------------------------------------------------------------------------------------
  | U2      | Name        | HDR1 | | HDR1 | Name         | Name Change | U1D
--|---------|-------------|------| |------|--------------|-------------|-------------------------------------------------------------------
  | 1 (TXD) | PQ4/FTDI-RX | 17   | | 17   | HDR-UART0_RX | UART1-RX    | A13 (PQ4)
  | 5 (RXD) | PQ5/FTDI-TX | 18   | | 18   | HDR-UART0_TX | UART1-TX    | W12 (PQ5)
******************************************************************************************************************************************/

// Include Files
#include "UART1.h"
#include "Buffers.h"
#include "LED.h"
//#include "USB_Driver.h"
//#include "Utils_objects.h"
//#include "Utils_resources.h"
#include "stdint.h"
#include "CC3000.h"
#include "__Net_Wireless_CC3000.h"
#include "RTCC.h"
#include "__Time.h"
#pragma funcall UART1_Rx1, UART1_Rx2
#pragma funcall UART1_Tx1

// Variables
char RxIn;                   // Input pointer to the next byte in receive FIFO
char RxOut;                  // Output pointer to the next byte in receive FIFO
char RxQueue[rxqsize];       // Re-circulating receive FIFO

char RcvBuf[RcvBufSize];     // Receive messages are assembled in this buffer
char RcvBufPtr;              // Pointer to the next free byte in RcvBuf


char TxIn;                   // Input pointer next element in the transmit FIFO
char TxOut;                  // Output pointer to the next element in the transmit FIFO
char *TxQueue[txqsize];      // Queue of message pointers in the transmit FIFO

char *XmtBuf;                // Pointer to the current message being transmitted
char XmtBufPtr;              // Points to a byte in transmit buffer

tScanResult myScanResult;
uint32_t IntervalList[16];

extern uint32_t  myIpAddr[];
extern unsigned char RTC_State;

// The receive message routine has 2 states
tPointerFcn UART1_RxFuncPtr = UART1_Rx1;  // Start in State 1

// This is not needed but included for symetry with the receiver
tPointerFcn UART1_TxFuncPtr = UART1_Tx1;  // Start in State 1


extern uint8_t State;

TimeStruct DateAndTime;



/******************************************************************************************************************************************
  Hardware Level Functions
  Interrupt Handler
******************************************************************************************************************************************/
void UART1_Interrupt() iv IVT_INT_UART1 ics ICS_AUTO {
     char R_data;        // Receive data byte
     char T_data;        // Transmit data byte

     // Do this segment if the receive holding register is full and the receiver is enabled
     if (UART_FR_RXFF_UART1_FR_bit && UART_IM_RXIM_UART1_IM_bit) {

         //LED_Set_Green();                // Diagnostic
         R_data = UART1_DR;              // Get the byte from receive holding register
         RxQueue[RxIn++] = R_data;       // Put data byte in re-circulating buffer, increment pointer
         if (RxIn > (rxqsize-1))         // Adjust pointer if necessary
             RxIn = 0;
     }

     // Do this segment if the thransmit holding register is empty and the transmitter is enabled
     if (UART_FR_TXFE_UART1_FR_bit && UART_IM_TXIM_UART1_IM_bit) {

         T_data = XmtBuf[XmtBufPtr++];   // Get a byte from the transmit buffer
         if (T_data == NUL) {            // If it is the end of message marker
             UART_IM_TXIM_UART1_IM_bit = 0;   // Disable transmitter interrupts
             //LED_Set_Blue();             // Diagnostic
         } else {
             UART1_DR = T_data;          // Transfer data byte to transmit register
         }
     }
}

/******************************************************************************************************************************************
  This routine initializes port Q, enables and sets the baud rate for the UART. 
******************************************************************************************************************************************/
void UART1_Enable(void) {

     SYSCTL_RCGCGPIO_R14_bit = 1;        // Enable and provide a clock to GPIO Port Q in Run mode.

     GPIO_PORTQ_DIR4_bit = 0;            // Set bit 4 as input (RX)
     GPIO_PORTQ_DIR5_bit = 1;            // Set bit 5 as output (TX)
     GPIO_PORTQ_AFSEL4_bit = 1;          // Alternate function for PQ4 (UART1-RX)
     GPIO_PORTQ_AFSEL5_bit = 1;          // Alternate function for PQ5 (UART1-TX)
     GPIO_PORTQ_PCTL |= 0x00110000;      // Select UART1 Rx and Tx
     GPIO_PORTQ_DEN |= 0x30;              // Enable PQ4 and PQ5 digital function

     SYSCTL_RCGCUART_R1_bit = 1;         // Enable and provide a clock to UART module 1 in Run mode.

     // BAUD register formulas and calculations
     // BRD = BRDI + BRDF = UARTSysClk / (ClkDiv * Baud Rate) = 120000000 / (16 * 9600) = 781.2500
     // UARTIBRD[DIVINT] = BRDI = INT(781.2500) = 781 = 0x030D
     // UARTFBRD[DIVFRAC] = INT(BRDF * 64 + 0.5) = INT(0.2500 * 64 + 0.5) = 16 = 0x10
     
     UART_CTL_UARTEN_UART1_CTL_bit = 1;  // Enable UART
     
     UART1_IBRD = 0x030D;                // Set integer portion
     UART1_FBRD = 0x10;                  // Set fraction portion
     UART1_LCRH = 0x60;                  // 8-bit data, 1-bit stop, no parity

     UART_CTL_TXE_UART1_CTL_bit = 1;     // This bit should already be on after reset
     UART_CTL_RXE_UART1_CTL_bit = 1;     // This bit should already be on after reset
     
     UART_ICR_RXIC_UART1_ICR_bit = 1;    // Clear raw interrupt
     //UART_ICR_TXIC_UART1_ICR_bit = 1;    // Clear raw interrupt
     
     NVIC_EN0_INT6_bit = 1;              // Enable UART1 interrupts
     UART1_DR = 0x00;                    // Kick start the transmitter. Don't know why this is necessary.
}

/******************************************************************************************************************************************
  This routine starts the receiver. 
******************************************************************************************************************************************/
void UART1_StartRx(void) {
     UART_IM_RXIM_UART1_IM_bit = 1;      // Enable receiver interrupts
}

/******************************************************************************************************************************************
  This routine starts the transmitter. 
******************************************************************************************************************************************/
void UART1_StartTx(void) {
     UART_IM_TXIM_UART1_IM_bit = 1;      // Enable transmitter interrupts
}

/******************************************************************************************************************************************
  This routine returns 0 if transmitter is busy, 1 if it is free. 
******************************************************************************************************************************************/
char UART1_CheckFree(void) {
      if (UART_IM_TXIM_UART1_IM_bit)     // UART is busy if transmitter interrupts are enabled
          return 0;
      else
          return 1;
}
//-End hardware level functions------------------------------------------------------------------------------------------------------------



/******************************************************************************************************************************************
  UART1 Software Functions
  Initialize UART
******************************************************************************************************************************************/
void UART1_Initialize(void) {
     UART1_RxQueueInit();                // Set up receiver queue pointers
     UART1_TxQueueInit();                // Set up transmitter queue pointers

     UART1_Enable();                     // Do hardware enabling
     UART1_StartRx();                    // Start the receiver
}

// The receive portion ********************************************************************************************************************
/******************************************************************************************************************************************
  This routine initializes the receive buffer in and out pointers. 
******************************************************************************************************************************************/
void UART1_RxQueueInit(void) {
     RxIn = RxOut = 0;                 // Set pointers to 0
     RcvBufPtr = 0;
}

/******************************************************************************************************************************************
  This routine retrieves a byte from the re-circulating buffer if present. Returns 0 if no data is
  available. Returns 1 if a data byte has been transferred successfully. The pointer to the data
  byte is passed to the routine. 
******************************************************************************************************************************************/
char UART1_GetChar(char *rxbyte) {
     if (RxIn != RxOut) {              // If there is a character in the buffer
        *rxbyte = RxQueue[RxOut++];    // Extract the character and update the pointer
        if (RxOut > (rxqsize-1))       // Check for wrap
           RxOut = 0;
        return 1;
     }else{
        return 0;                      // No data available
     }
}

/******************************************************************************************************************************************
  UART_Rx State Machine. Call this function in the main loop. 
******************************************************************************************************************************************/
void UART1_Rx(void) {
     UART1_RxFuncPtr();    // Use the function pointer to go to a state
}

/******************************************************************************************************************************************
  UART_Rx State 1: Accumulate received characters in a buffer until a Line Feed character is
  received then insert a NUL character into the buffer and go to State 2. 
******************************************************************************************************************************************/
void UART1_Rx1(void) {
     char rxchar;

     if (UART1_GetChar(&rxchar)) {
         switch (rxchar) {

             case NUL:         // Discard NUL character
                 break;

             case '\r':        // Discard return character
                 break;

             case '\n':
                 RcvBuf[RcvBufPtr++] = rxchar;       // Put received character into buffer
                 RcvBuf[RcvBufPtr] = 0;              // End marker
                 RcvBufPtr = 0;                      // Point to start of buffer
                 UART1_RxFuncPtr = UART1_Rx2;        // Go to state 2
                 break;

             default:
                 if (RcvBufPtr < (RcvBufSize-2))     // Prevents buffer overrun
                     RcvBuf[RcvBufPtr++] = rxchar;   // Put character in buffer
                 break;
         }
     }
}

/******************************************************************************************************************************************
  UART_Rx State 2: Execute the command indicated by the first byte in the buffer then return to
  State 1. 
******************************************************************************************************************************************/
void UART1_Rx2(void) {
     char *BufNum;
     char Ver[4];
     uint32_t Value;

     switch (RcvBuf[0]) {  // Branch on command byte

        // Change to suit application
        case 'a':
           UART1_PutQueue("Command a received\r\n");
           break;

        case 'b':
           SPI1_Initialize();
           Net_Wireless_CC3000_Init();
           Net_Wireless_CC3000_regAsynHandler(Net_Wireless_CC3000_UsynchCallback);
           NVIC_IntEnable(IVT_INT_GPIOB);
           Net_Wireless_CC3000_Start(0);
           
           BufNum = GetBuffer();
           if (Net_Wireless_CC3000_NvmemReadSpVer(&Ver) == 0)
               sprintf(BufNum,"Version: %d.%d\r\n",Ver[0],Ver[1]);
           else
               strcpy(BufNum,"Error\r\n");

           UART1_PutQueue(BufNum);
           Net_Wireless_CC3000_Stop();
           break;

        // Commands 'c' and 'd' should be used together
        case 'c': // Set scan parameters for AP identification. Send this command first
           UART1_PutQueue("CC3000 initializing\r\n");
           SPI1_Initialize();
           Net_Wireless_CC3000_Init();
           Net_Wireless_CC3000_regAsynHandler(Net_Wireless_CC3000_UsynchCallback);
           NVIC_IntEnable(IVT_INT_GPIOB);
           Net_Wireless_CC3000_Start(0);
           
           if (Net_Wireless_CC3000_SetScanParams(1, 100, 200, 5, 0x1fff, -75, 0, 205, 0) == 0)
                   UART1_PutQueue("Success\r\n");
               else
                   UART1_PutQueue("Failed\r\n");
           break;
        
        case 'd': // Get scan result. Send this command multiple times to display all available APs. A blank line indicates end of AP list
           BufNum = GetBuffer();
           if (Net_Wireless_CC3000_GetScanResult(0, &myScanResult) == 0) {
               strncpy(BufNum,myScanResult.ssidName,myScanResult.ssidNameLength);
               strcat(BufNum,"\r\n");
           } else
               strcpy(BufNum,"Error\r\n");

           UART1_PutQueue(BufNum);
           break;
        
        case 'e': // Unused
           UART1_PutQueue("Unused command\r\n");
           break;

        case 'f': // Unused
           UART1_PutQueue("Unused command\r\n");
           break;
        
        case 'g': // Display initial IP address. Normally set to 0.0.0.0 and dhcp will assign it later
           BufNum = GetBuffer();
           sprintf(BufNum,"My IP: %d.%d.%d.%d\r\n",myIpAddr[0],myIpAddr[1],myIpAddr[2],myIpAddr[3]);
           UART1_PutQueue(BufNum);
           break;

        case 'h': // Display DHCP assigned addresses
           DispDHCP();
           break;
        
        case 'i': // Unused
           UART1_PutQueue("Unused command\r\n");
           break;
        
        case 'j': // Unused
           UART1_PutQueue("Unused command\r\n");
           break;
        
        case 'k': // Display a register
           SPI1_Initialize();
           BufNum = GetBuffer();
           sprintf(BufNum,"Register: %08LX\r\n",SSI1_CR0);
           UART1_PutQueue(BufNum);
           break;
        
        case 'l': // Show CC3000 State machine State if hung-up
           BufNum = GetBuffer();
           sprintf(BufNum,"Current State of CC3000 state machine: %d\r\n",State);
           UART1_PutQueue(BufNum);
           BufNum = GetBuffer();
           sprintf(BufNum,"Current State of RTCC state machine: %d\r\n",RTC_State);
           UART1_PutQueue(BufNum);
           break;
        
        case 'm': // Run State machine to display all APs in the area
           State = 33;
           
           break;
        
        case 'n': // Run State machine with diagnostic displays
           State = 38;
           break;

        // The following two commands should be used one after the other
        case 'o': // Run State machine to obtain date and time from server and store into RTC calendar registers
           RTC_State = 1;
           break;
        
        case 'p': // Read calendar registers and display
           RTC_State = 16;
           break;
        
        case 'r': // Display contror register
           BufNum = GetBuffer();
           sprintf(BufNum,"Register: %08LX\r\n",HIB_CTL);
           UART1_PutQueue(BufNum);
           break;
        
        case 's': // Run State machine to contact time server only
           State = 1;
           break;
        
        case 't': // Debug
           Value = Net_Wireless_CC3000_Time;
           BufNum = GetBuffer();
           sprintf(BufNum,"Time: %d\r\n",Value);
           UART1_PutQueue(BufNum);
           break;
        
        case 'u': // Debug
           Value = RTCC_Read(&DateAndTime);
           BufNum = GetBuffer();
           sprintf(BufNum,"Date: %d/%d/%d  Time: %d:%d:%d\r\n",DateAndTime.yy,DateAndTime.mo,DateAndTime.md,DateAndTime.hh,
                                                                            DateAndTime.mn,DateAndTime.ss);
           UART1_PutQueue(BufNum);
           break;
        
        case 'v': // Disable hybernate (RTC) circuits
           HibernateRTCDisable();
           break;
        
        case 'w': // Debug
           RTC_Init();
           break;
        
        case 'x': // Test: Write calendar registers with fixed data
           RTC_State = 18;
           break;
        
        case 'z':  // Microprocessor software reset
           SystemReset();
           break;

        default:
           BufNum = GetBuffer();
           sprintf(BufNum,"Unknown command: %c\r\n",RcvBuf[0]);
           UART1_PutQueue(BufNum);
           break;
     }
     UART1_RxFuncPtr = UART1_Rx1;  // Go back to state 1
}
//-----------------------------------------------------------------------------------------------------------------------------------------


// The transmit portion *******************************************************************************************************************
/******************************************************************************************************************************************
  This routine initializes the transmit buffer queue in and out pointers. 
******************************************************************************************************************************************/
void UART1_TxQueueInit(void) {
     TxIn = TxOut = 0;         // Set pointers to 0
     XmtBufPtr = 0;
}

/******************************************************************************************************************************************
  This routine makes an entry into the transmit queue of pointers and updates the in pointer to
  the next available location. 
******************************************************************************************************************************************/
void UART1_PutQueue(char *msg) {
     TxQueue[TxIn++] = msg;    // Put the message pointer into FIFO
     if (TxIn > (txqsize-1))   // Adjust FIFO in pointer
         TxIn = 0;
}

/******************************************************************************************************************************************
  This routine enters a pointer to a text message and starts the transmitter. 
******************************************************************************************************************************************/
void UART1_SendMsg(char *s) {
     XmtBuf = s;               // Pointer to the message for the interrupt handler
     XmtBufPtr = 0;            // Set character counter to start of text message
     UART1_StartTx();          // Turn on the transmitter
}

/******************************************************************************************************************************************
  UART1_Tx State Machine. Call this function in the main loop.
******************************************************************************************************************************************/
void UART1_Tx(void) {
     UART1_TxFuncPtr();    // Use the function pointer to go to a state
}

/******************************************************************************************************************************************
  This routine checks the transmit buffer queue and if it is not empty, it extracts the pointer
  to the message and sets up the transmitter. Call this function in the main loop.
******************************************************************************************************************************************/
void UART1_Tx1(void) {
     if (UART1_CheckFree()) {                 // Don't try to start transmitter if already transmitting
         if (TxIn != TxOut) {                 // If pointers are not the same there is something to send
             UART1_SendMsg(TxQueue[TxOut++]); // Hand off the message address and increment queue out pointer
             if (TxOut > (txqsize-1))         // Check for wrap
                 TxOut = 0;
         }
     }
}

/******************************************************************************************************************************************
  Send one message and wait for completion.
******************************************************************************************************************************************/
void UART1_Send(char *s) {
     EnableInterrupts();
     while (UART1_CheckFree == 0) {}
     UART1_PutQueue(s);
     UART1_Tx();
     while (UART1_CheckFree == 0) {}
}

/******************************************************************************************************************************************
  End of file.
******************************************************************************************************************************************/